module top(
        input               rstn,
        
        output              rx_error_all,
        output              ip_complete
    );

localparam      BIT_ONE             =   1'b1;
localparam      BIT_ZERO            =   1'b0;

wire            clk_in;
//IP configuration_vector
wire            mr_main_reset;
wire            mr_loopback;
wire            mr_an_enable;
wire            isolate;
wire            mr_restart_an;
wire            unidir_en;
wire            ps2;
wire            ps1;
wire            fd;
wire            hd;
wire            power_down;
wire    [ 1:0]  sgmii_phy_speed;
wire            sgmii_phy_duplex;
wire            sgmii_phy_link;

wire            ip_link_complete;

reg     [ 7:0]  ip_link_complete_cnt_tx;
reg             tx_mac_rst_n;

reg     [ 7:0]  ip_link_complete_cnt_rx;
reg             rx_mac_rst_n;

//IP TX PORT
wire            tx_mac_clk;
wire            tx_mac_valid;
wire    [ 7:0]  tx_mac_data;
wire            tx_mac_last;
wire            tx_mac_error;
wire            tx_mac_ready;

wire            tx_pause_req;
wire    [15:0]  tx_pause_val;
wire    [47:0]  tx_pause_source_addr;

//IP RX PORT
wire            rx_mac_clk;
wire            rx_mac_valid;
wire    [ 7:0]  rx_mac_data;
wire            rx_mac_last;
wire            rx_mac_error;

wire            rx_statistics_valid;
wire    [26:0]  rx_statistics_vector;
wire            rx_pause_req;
wire    [15:0]  rx_pause_val;

//IP MDIO/MDC PORT
wire    [14:0]  configuration_vector;
wire    [10:0]  status_vector;
wire    [15:0]  link_partner_ability_base;

OSC u_osc (.OSCOUT(clk_in) );
defparam u_osc.FREQ_DIV = 4;

//1000BASE-X 有MAC层
Serdes_Top u_Serdes_Top(
        .Giga_Serial_Ethernet_Top_clk_in_i                      (clk_in                     ),
        .Giga_Serial_Ethernet_Top_rstn_i                        (rstn                       ),
        .Giga_Serial_Ethernet_Top_signal_detect_i               (1'b1                       ),
    //config port
    //link_timer, multiplied by 32.768us
        .Giga_Serial_Ethernet_Top_basex_link_timer_i            (9'd305                     ),
        .Giga_Serial_Ethernet_Top_sgmii_link_timer_i            (9'd49                      ),
        .Giga_Serial_Ethernet_Top_basex_or_sgmii_i              (1'b0                       ),
        .Giga_Serial_Ethernet_Top_sgmii_mode_i                  (1'b0                       ),
        .Giga_Serial_Ethernet_Top_repeater_mode_i               (1'b0                       ),
    
        .Giga_Serial_Ethernet_Top_rx_mac_clk_o                  (rx_mac_clk                 ),
        .Giga_Serial_Ethernet_Top_rx_mac_valid_o                (rx_mac_valid               ),
        .Giga_Serial_Ethernet_Top_rx_mac_data_o                 (rx_mac_data                ),
        .Giga_Serial_Ethernet_Top_rx_mac_last_o                 (rx_mac_last                ),
        .Giga_Serial_Ethernet_Top_rx_mac_error_o                (rx_mac_error               ),
    
        .Giga_Serial_Ethernet_Top_rx_fcs_fwd_ena_i              (1'b0                       ),
        .Giga_Serial_Ethernet_Top_rx_jumbo_ena_i                (1'b0                       ),
        .Giga_Serial_Ethernet_Top_rx_statistics_valid_o         (rx_statistics_valid        ),
        .Giga_Serial_Ethernet_Top_rx_statistics_vector_o        (rx_statistics_vector       ),
        .Giga_Serial_Ethernet_Top_rx_pause_req_o                (rx_pause_req               ),
        .Giga_Serial_Ethernet_Top_rx_pause_val_o                (rx_pause_val               ),
    
        .Giga_Serial_Ethernet_Top_tx_mac_clk_o                  (tx_mac_clk                 ),
        .Giga_Serial_Ethernet_Top_tx_mac_valid_i                (tx_mac_valid               ),
        .Giga_Serial_Ethernet_Top_tx_mac_data_i                 (tx_mac_data                ),
        .Giga_Serial_Ethernet_Top_tx_mac_last_i                 (tx_mac_last                ),
        .Giga_Serial_Ethernet_Top_tx_mac_error_i                (tx_mac_error               ),
        .Giga_Serial_Ethernet_Top_tx_mac_ready_o                (tx_mac_ready               ),
        .Giga_Serial_Ethernet_Top_tx_collision_o                (                           ),
        .Giga_Serial_Ethernet_Top_tx_retransmit_o               (                           ),

        .Giga_Serial_Ethernet_Top_tx_pause_req_i                (tx_pause_req               ),
        .Giga_Serial_Ethernet_Top_tx_pause_val_i                (tx_pause_val               ),
        .Giga_Serial_Ethernet_Top_tx_pause_source_addr_i        (tx_pause_source_addr       ),
        .Giga_Serial_Ethernet_Top_tx_ifg_delay_ena_i            (1'b0                       ),
        .Giga_Serial_Ethernet_Top_tx_ifg_delay_i                (8'd8                       ),
        .Giga_Serial_Ethernet_Top_tx_fcs_fwd_ena_i              (1'b0                       ),
        .Giga_Serial_Ethernet_Top_tx_statistics_valid_o         (                           ),
        .Giga_Serial_Ethernet_Top_tx_statistics_vector_o        (                           ),    
    //mdio
        .Giga_Serial_Ethernet_Top_miim_hs_clk_i                 (1'd0                       ), // input clk
        .Giga_Serial_Ethernet_Top_miim_phyad_i                  (5'd0                       ), // input [4:0] miim_phyad
        .Giga_Serial_Ethernet_Top_miim_regad_i                  (5'd0                       ), // input [4:0] miim_regad
        .Giga_Serial_Ethernet_Top_miim_wrdata_i                 (16'd0                      ), // input [15:0] miim_wrdata
        .Giga_Serial_Ethernet_Top_miim_wren_i                   (1'd0                       ), // input miim_wren
        .Giga_Serial_Ethernet_Top_miim_rden_i                   (1'd0                       ), // input miim_rden
        .Giga_Serial_Ethernet_Top_miim_rddata_o                 (                           ), // output [15:0] miim_rddata
        .Giga_Serial_Ethernet_Top_miim_rddata_valid_o           (                           ), // output miim_rddata_valid
        .Giga_Serial_Ethernet_Top_miim_busy_o                   (                           ), // output miim_busy
        .Giga_Serial_Ethernet_Top_mdc_o                         (                           ), // output mdc
        .Giga_Serial_Ethernet_Top_mdio_in_i                     (1'd0                       ), // input mdio_in
        .Giga_Serial_Ethernet_Top_mdio_out_o                    (                           ), // output mdio_out
        .Giga_Serial_Ethernet_Top_mdio_oen_o                    (                           ), // output mdio_oen
    //pcs config
        .Giga_Serial_Ethernet_Top_configuration_vector_i        (configuration_vector       ),
        .Giga_Serial_Ethernet_Top_status_vector_o               (status_vector              ),
        .Giga_Serial_Ethernet_Top_link_partner_ability_base_o   (link_partner_ability_base  )
    
    );

assign  configuration_vector    = { sgmii_phy_link,
                                    sgmii_phy_duplex,
                                    sgmii_phy_speed,
                                    hd,
                                    fd,
                                    ps2,
                                    ps1,
                                    power_down,
                                    unidir_en,
                                    mr_restart_an,
                                    isolate,
                                    mr_an_enable,
                                    mr_loopback,
                                    mr_main_reset};

assign  sgmii_phy_link          =   BIT_ZERO;
assign  sgmii_phy_duplex        =   BIT_ZERO;
assign  sgmii_phy_speed         =   {BIT_ZERO,BIT_ZERO};
assign  hd                      =   BIT_ONE;
assign  fd                      =   BIT_ONE;
assign  ps2                     =   BIT_ZERO;
assign  ps1                     =   BIT_ZERO;
assign  power_down              =   BIT_ZERO;
assign  unidir_en               =   BIT_ZERO;
assign  mr_restart_an           =   BIT_ZERO;
assign  isolate                 =   BIT_ZERO;
assign  mr_an_enable            =   BIT_ONE;
assign  mr_loopback             =   BIT_ZERO;
assign  mr_main_reset           =   BIT_ZERO;

assign  ip_link_complete        =   mr_an_enable ? (status_vector[2] && status_vector[1] && status_vector[0]) : (status_vector[1] && status_vector[0]);

assign  ip_complete             =   ip_link_complete;

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        ip_link_complete_cnt_rx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_rx == 8'hff) begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx;
        end else begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_rx <=  8'd0;
    end
end

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        rx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_rx == 8'hff) ) begin
        rx_mac_rst_n    <=  1'b1;
    end else begin
        rx_mac_rst_n    <=  1'b0;  
    end
end

always @(posedge tx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        ip_link_complete_cnt_tx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_tx == 8'hff) begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx;
        end else begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_tx <=  8'd0;
    end
end

always @(posedge tx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        tx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_tx == 8'hff) ) begin
        tx_mac_rst_n    <=  1'b1;
    end else begin
        tx_mac_rst_n    <=  1'b0;  
    end
end

mac_tx_model u_mac_tx_model(
        .tx_mac_clk                 (tx_mac_clk                 ),
        .tx_mac_rst_n               (tx_mac_rst_n               ),

    //IP TX PORT
        .tx_mac_valid               (tx_mac_valid               ),
        .tx_mac_data                (tx_mac_data                ),
        .tx_mac_last                (tx_mac_last                ),
        .tx_mac_error               (tx_mac_error               ),
        .tx_mac_ready               (tx_mac_ready               ),

        .tx_pause_req               (tx_pause_req               ),
        .tx_pause_val               (tx_pause_val               ),
        .tx_pause_source_addr       (tx_pause_source_addr       )
    );

mac_rx_model u_mac_rx_model(
        .rx_mac_clk                 (rx_mac_clk                 ),
        .rx_mac_rst_n               (rx_mac_rst_n               ),

    //IP TX PORT
        .rx_mac_valid               (rx_mac_valid               ),
        .rx_mac_data                (rx_mac_data                ),
        .rx_mac_last                (rx_mac_last                ),
        .rx_mac_error               (rx_mac_error               ),

        .rx_statistics_valid        (rx_statistics_valid        ),
        .rx_statistics_vector       (rx_statistics_vector       ),
        .rx_pause_req               (rx_pause_req               ),
        .rx_pause_val               (rx_pause_val               ),

    //CHECK RESULT
        .rx_error_all               (rx_error_all               )
    );

endmodule