`timescale 1ns/1ps

module tb;

    GSR GSR(1'b1);

localparam      IDLE                =   3'b000;   // 初始状态
localparam      UPDATE_PAUSE        =   3'b110;   // 计算下一帧PAUSE帧需要的信息

localparam      BIT_ONE             =   1'b1;
localparam      BIT_ZERO            =   1'b0;

//
reg             clk_in;
reg             rstn;
reg             pcs_tx_o_fabric_clk;
reg             pcs_rx_o_fabric_clk;

//IP configuration_vector
reg             mr_main_reset;
reg             mr_loopback;
reg             mr_an_enable;
reg             isolate;
reg             mr_restart_an;
reg             unidir_en;
reg             ps2;
reg             ps1;
reg             fd;
reg             hd;
reg             power_down;
reg     [ 1:0]  sgmii_phy_speed;
reg             sgmii_phy_duplex;
reg             sgmii_phy_link;

wire            ip_link_complete;           // PHY LINK UP

reg     [ 7:0]  ip_link_complete_cnt_tx;
reg             tx_mac_rst_n;

reg     [ 7:0]  ip_link_complete_cnt_rx;
reg             rx_mac_rst_n;

//IP TX PORT
wire            tx_mac_clk;
wire            tx_mac_valid;
wire    [ 7:0]  tx_mac_data;
wire            tx_mac_last;
wire            tx_mac_error;
wire            tx_mac_ready;

wire            tx_pause_req;
wire    [15:0]  tx_pause_val;
wire    [47:0]  tx_pause_source_addr;

wire    [ 9:0]  serdes_txdata;
//IP RX PORT
wire            rx_mac_clk;
wire            rx_mac_valid;
wire    [ 7:0]  rx_mac_data;
wire            rx_mac_last;
wire            rx_mac_error;

wire            rx_statistics_valid;
wire    [26:0]  rx_statistics_vector;
wire            rx_pause_req;
wire    [15:0]  rx_pause_val;

wire    [10:0]  serdes_rxdata;
//IP MDIO/MDC PORT
wire    [14:0]  configuration_vector;
wire    [10:0]  status_vector;
wire    [15:0]  link_partner_ability_base;

//CHECK RESULT
wire            rx_error_all;           // RX端接收的任何数据出现异常

reg     [15:0]  cycle_all_time_cnt = 16'd0;

Giga_Serial_Ethernet_Top u_Giga_Serial_Ethernet_Top(
        .serdes_fabric_ctrl_o           (),
        .serdes_fabric_ctrl_h_o         (),
        .serdes_fabric_rstn_o           (),
        .serdes_fabric_tx_clk_o         (),
        .serdes_fabric_rx_clk_o         (),
        .serdes_pcs_tx_rst_o            (),
        .serdes_pcs_rx_rst_o            (),
        .serdes_fabric_tx_vld_o         (),
        .serdes_rxfifo_rd_en_o          (),
        .serdes_txdata_o                (serdes_txdata              ),
        .rx_mac_clk_o                   (rx_mac_clk                 ),
        .rx_mac_valid_o                 (rx_mac_valid               ),
        .rx_mac_data_o                  (rx_mac_data                ),
        .rx_mac_last_o                  (rx_mac_last                ),
        .rx_mac_error_o                 (rx_mac_error               ),
        .tx_mac_clk_o                   (tx_mac_clk                 ),
        .tx_mac_ready_o                 (tx_mac_ready               ),
        .tx_collision_o                 (),
        .tx_retransmit_o                (),
        .rx_statistics_valid_o          (rx_statistics_valid        ),
        .rx_statistics_vector_o         (rx_statistics_vector       ),
        .rx_pause_req_o                 (rx_pause_req               ),
        .rx_pause_val_o                 (rx_pause_val               ),
        .tx_statistics_valid_o          (),
        .tx_statistics_vector_o         (),
        .miim_rddata_o                  (),
        .miim_rddata_valid_o            (),
        .miim_busy_o                    (),
        .mdc_o                          (),
        .mdio_out_o                     (),
        .mdio_oen_o                     (),
        .status_vector_o                (status_vector              ),
        .link_partner_ability_base_o    (link_partner_ability_base  ),
        .clk_in_i                       (clk_in                     ),
        .rstn_i                         (rstn                       ),
        .serdes_pcs_tx_clk_i            (pcs_tx_o_fabric_clk        ),
        .serdes_pcs_rx_clk_i            (pcs_rx_o_fabric_clk        ),
        .serdes_qpll1_ok_i              (BIT_ONE                    ),
        .serdes_qpll0_ok_i              (BIT_ONE                    ),
        .serdes_cpll_ok_i               (BIT_ONE                    ),
        .serdes_pma_rx_lock_i           (BIT_ONE                    ),
        .serdes_tx_fifo_afull_i         (BIT_ZERO                   ),
        .serdes_rxfifo_aempty_i         (BIT_ZERO                   ),
        .serdes_rxdata_i                (serdes_rxdata              ),
        .signal_detect_i                (1'b1                       ),
        .basex_link_timer_i             (9'd5                       ),
        .sgmii_link_timer_i             (9'd20                      ),
        .basex_or_sgmii_i               (1'b0                       ),
        .sgmii_mode_i                   (1'b0                       ),
        .repeater_mode_i                (1'b0                       ),
        .tx_mac_valid_i                 (tx_mac_valid               ),
        .tx_mac_data_i                  (tx_mac_data                ),
        .tx_mac_last_i                  (tx_mac_last                ),
        .tx_mac_error_i                 (tx_mac_error               ),
        .rx_fcs_fwd_ena_i               (1'd0                       ),
        .rx_jumbo_ena_i                 (1'd0                       ),
        .tx_pause_req_i                 (tx_pause_req               ),
        .tx_pause_val_i                 (tx_pause_val               ),
        .tx_pause_source_addr_i         (tx_pause_source_addr       ),
        .tx_ifg_delay_ena_i             (1'd0                       ),
        .tx_ifg_delay_i                 (8'd0                       ),
        .tx_fcs_fwd_ena_i               (1'd0                       ),
        .miim_hs_clk_i                  (1'd0                       ),
        .miim_phyad_i                   (5'd0                       ),
        .miim_regad_i                   (5'd0                       ),
        .miim_wrdata_i                  (16'd0                      ),
        .miim_wren_i                    (1'd0                       ),
        .miim_rden_i                    (1'd0                       ),
        .mdio_in_i                      (1'd0                       ),
        .configuration_vector_i         (configuration_vector       )
    );

assign  serdes_rxdata   =   {2'b00,serdes_txdata[8:0]};

assign  configuration_vector    = { sgmii_phy_link,
                                    sgmii_phy_duplex,
                                    sgmii_phy_speed,
                                    hd,
                                    fd,
                                    ps2,
                                    ps1,
                                    power_down,
                                    unidir_en,
                                    mr_restart_an,
                                    isolate,
                                    mr_an_enable,
                                    mr_loopback,
                                    mr_main_reset};

initial begin
    sgmii_phy_link          =   BIT_ZERO;
    sgmii_phy_duplex        =   BIT_ZERO;
    sgmii_phy_speed         =   {BIT_ZERO,BIT_ZERO};
    hd                      =   BIT_ONE;
    fd                      =   BIT_ONE;
    ps2                     =   BIT_ZERO;
    ps1                     =   BIT_ZERO;
    power_down              =   BIT_ZERO;
    unidir_en               =   BIT_ZERO;
    mr_restart_an           =   BIT_ZERO;
    isolate                 =   BIT_ZERO;
    mr_an_enable            =   BIT_ONE;
    mr_loopback             =   BIT_ZERO;
    mr_main_reset           =   BIT_ZERO;
end

assign  ip_link_complete        =   mr_an_enable ? (status_vector[2] && status_vector[1] && status_vector[0]) : (status_vector[1] && status_vector[0]);

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        ip_link_complete_cnt_rx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_rx == 8'hff) begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx;
        end else begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_rx <=  8'd0;
    end
end

always @(posedge rx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        rx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_rx == 8'hff) ) begin
        rx_mac_rst_n    <=  1'b1;
    end else begin
        rx_mac_rst_n    <=  1'b0;  
    end
end

always @(posedge tx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        ip_link_complete_cnt_tx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_tx == 8'hff) begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx;
        end else begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_tx <=  8'd0;
    end
end

always @(posedge tx_mac_clk or negedge rstn) begin
    if(!rstn) begin
        tx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_tx == 8'hff) ) begin
        tx_mac_rst_n    <=  1'b1;
    end else begin
        tx_mac_rst_n    <=  1'b0;  
    end
end

mac_tx_model u_mac_tx_model(
        .tx_mac_clk                     (tx_mac_clk                 ), //
        .tx_mac_rst_n                   (tx_mac_rst_n               ), //

    //IP TX PORT
        .tx_mac_valid                   (tx_mac_valid               ), // 发送以太网帧的使能
        .tx_mac_data                    (tx_mac_data                ), // 发送以太网帧的数据
        .tx_mac_last                    (tx_mac_last                ), // 发送以太网帧的最后字节
        .tx_mac_error                   (tx_mac_error               ),  // 发送以太网帧的错误
        .tx_mac_ready                   (tx_mac_ready               ), // 发送以太网帧的握手

        .tx_pause_req                   (tx_pause_req               ), // 发送PAUSE帧的请求
        .tx_pause_val                   (tx_pause_val               ), // 发送PASUE帧的2字节暂停时间
        .tx_pause_source_addr           (tx_pause_source_addr       )  // 发送PASUE帧的6字节源地址
    );

mac_rx_model u_mac_rx_model(
        .rx_mac_clk                     (rx_mac_clk                 ),
        .rx_mac_rst_n                   (rx_mac_rst_n               ),

    //IP TX PORT
        .rx_mac_valid                   (rx_mac_valid               ), // 接收以太网帧的使能
        .rx_mac_data                    (rx_mac_data                ), // 接收以太网帧的数据
        .rx_mac_last                    (rx_mac_last                ), // 接收以太网帧的最后字节
        .rx_mac_error                   (rx_mac_error               ), // 接收以太网帧的错误

        .rx_statistics_valid            (rx_statistics_valid        ), // 接收统计信息的有效
        .rx_statistics_vector           (rx_statistics_vector       ), // 接收统计信息
        .rx_pause_req                   (rx_pause_req               ), // 接收PAUSE帧的请求
        .rx_pause_val                   (rx_pause_val               ), // 接收PASUE帧的2字节暂停时间

    //CHECK RESULT
        .rx_error_all                   (rx_error_all               )  // RX端接收的任何数据出现异常
    );

    initial begin
        clk_in = 0;
        forever begin
            #10;
            clk_in = ~clk_in;
        end
    end
    
    initial begin
        pcs_tx_o_fabric_clk = 0;
        forever begin
            #4
            pcs_tx_o_fabric_clk = ~pcs_tx_o_fabric_clk;
        end
    end

    initial begin
        pcs_rx_o_fabric_clk = 0;
        forever begin
            #4;
            pcs_rx_o_fabric_clk = ~pcs_rx_o_fabric_clk;
        end
    end

    initial begin
        rstn = 0;
        #100;
        rstn = 1;
    end

    initial begin
        wait(rstn == 1'b1);

        wait(ip_link_complete);
        $display("--------------%0tus:Auto-Negotiation is complete-------------------",$time);

        repeat(50) @(posedge clk_in);
        $display("--------------%0tus:start IP Transceiver test Continuously-------------------",$time);
        while(cycle_all_time_cnt < 16'd2000) begin
            @(posedge rx_mac_clk);
            if ( (u_mac_rx_model.cur_state != IDLE) && (u_mac_rx_model.next_state == IDLE) ) begin
                cycle_all_time_cnt  <=  cycle_all_time_cnt + 4'd1;
            end else begin
                cycle_all_time_cnt  <=  cycle_all_time_cnt;
            end
        end
        $display("--------------%0tus:end IP Transceiver test Continuously-------------------",$time);

        #1000;
        
        $finish;
    end

    initial begin
        $fsdbDumpfile("tb.fsdb");
        $fsdbDumpvars;
    end

endmodule